﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.ParameterVectorEncoding;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Programmable {
  [Item("ParameterVector Crossover", "Applies different crossovers to cross a parameter vector.")]
  [StorableClass]
  public class ParameterVectorCrossover : InstrumentedOperator, ICrossover {

    public ILookupParameter<ParameterVector> ChildParameter {
      get { return (ILookupParameter<ParameterVector>)Parameters["Child"]; }
    }
    public IScopeTreeLookupParameter<ParameterVector> ParentsParameter {
      get { return (IScopeTreeLookupParameter<ParameterVector>)Parameters["Parents"]; }
    }

    public ILookupParameter<BinaryVector> BinaryVectorParameter {
      get { return (ILookupParameter<BinaryVector>)Parameters["BinaryVector"]; }
    }

    public ILookupParameter<IntegerVector> IntegerVectorParameter {
      get { return (ILookupParameter<IntegerVector>)Parameters["IntegerVector"]; }
    }

    public ILookupParameter<RealVector> RealVectorParameter {
      get { return (ILookupParameter<RealVector>)Parameters["RealVector"]; }
    }

    public ILookupParameter<Permutation> PermutationParameter {
      get { return (ILookupParameter<Permutation>)Parameters["Permutation"]; }
    }

    [StorableConstructor]
    protected ParameterVectorCrossover(bool deserializing) : base(deserializing) { }
    protected ParameterVectorCrossover(ParameterVectorCrossover original, Cloner cloner)
      : base(original, cloner) { }
    public ParameterVectorCrossover() {
      Parameters.Add(new LookupParameter<ParameterVector>("Child", "The parameter vector of the child.", "ParameterVector"));
      Parameters.Add(new ScopeTreeLookupParameter<ParameterVector>("Parents", "The parameter vector of the parents.", "ParameterVector"));
      Parameters.Add(new LookupParameter<BinaryVector>("BinaryVector", "The binary vector that links into the parameter vector."));
      Parameters.Add(new LookupParameter<IntegerVector>("IntegerVector", "The integer vector that links into the parameter vector."));
      Parameters.Add(new LookupParameter<RealVector>("RealVector", "The real vector that links into the parameter vector."));
      Parameters.Add(new LookupParameter<Permutation>("Permutation", "The permutation that links into the parameter vector."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterVectorCrossover(this, cloner);
    }

    public override IOperation InstrumentedApply() {
      var child = (ParameterVector)ParentsParameter.ActualValue[0].Clone();
      child.BooleanParameters = BinaryVectorParameter.ActualValue;
      child.IntegerParameters = IntegerVectorParameter.ActualValue;
      child.RealParameters = RealVectorParameter.ActualValue;
      child.PermutationParameter = PermutationParameter.ActualValue;
      ChildParameter.ActualValue = child;
      return base.InstrumentedApply();
    }
  }
}
